@@ -24,10 +24,11 @@ import asyncio
 import ipaddress
 import random
 from functools import lru_cache
-from typing import TYPE_CHECKING, Dict, List, Optional, Union, cast
+from typing import TYPE_CHECKING, Dict, List, Optional, Set, Union, cast
 
 from .._dns import (
     DNSAddress,
+    DNSNsec,
     DNSPointer,
     DNSQuestionType,
     DNSRecord,
@@ -47,19 +48,24 @@ from .._utils.name import service_type_name
 from .._utils.net import IPVersion, _encode_address
 from .._utils.time import current_time_millis, millis_to_seconds
 from ..const import (
+    _ADDRESS_RECORD_TYPES,
     _CLASS_IN,
     _CLASS_UNIQUE,
     _DNS_HOST_TTL,
     _DNS_OTHER_TTL,
     _FLAGS_QR_QUERY,
     _LISTENER_TIME,
+    _MDNS_PORT,
     _TYPE_A,
     _TYPE_AAAA,
+    _TYPE_NSEC,
     _TYPE_PTR,
     _TYPE_SRV,
     _TYPE_TXT,
 )
 
+_IPVersion_All_value = IPVersion.All.value
+_IPVersion_V4Only_value = IPVersion.V4Only.value
 # https://datatracker.ietf.org/doc/html/rfc6762#section-5.2
 # The most common case for calling ServiceInfo is from a
 # ServiceBrowser. After the first request we add a few random
@@ -242,14 +248,15 @@ class ServiceInfo(RecordUpdateListener):
         This means the first address will always be the most recently added
         address of the given IP version.
         """
-        if version == IPVersion.V4Only:
+        version_value = version.value
+        if version_value == _IPVersion_All_value:
+            return [
+                *(addr.packed for addr in self._ipv4_addresses),
+                *(addr.packed for addr in self._ipv6_addresses),
+            ]
+        if version_value == _IPVersion_V4Only_value:
             return [addr.packed for addr in self._ipv4_addresses]
-        if version == IPVersion.V6Only:
-            return [addr.packed for addr in self._ipv6_addresses]
-        return [
-            *(addr.packed for addr in self._ipv4_addresses),
-            *(addr.packed for addr in self._ipv6_addresses),
-        ]
+        return [addr.packed for addr in self._ipv6_addresses]
 
     def ip_addresses_by_version(
         self, version: IPVersion
@@ -262,11 +269,17 @@ class ServiceInfo(RecordUpdateListener):
         This means the first address will always be the most recently added
         address of the given IP version.
         """
-        if version == IPVersion.V4Only:
+        return self._ip_addresses_by_version_value(version.value)
+
+    def _ip_addresses_by_version_value(
+        self, version_value: int
+    ) -> Union[List[ipaddress.IPv4Address], List[ipaddress.IPv6Address], List[ipaddress._BaseAddress]]:
+        """Backend for addresses_by_version that uses the raw value."""
+        if version_value == _IPVersion_All_value:
+            return [*self._ipv4_addresses, *self._ipv6_addresses]
+        if version_value == _IPVersion_V4Only_value:
             return self._ipv4_addresses
-        if version == IPVersion.V6Only:
-            return self._ipv6_addresses
-        return [*self._ipv4_addresses, *self._ipv6_addresses]
+        return self._ipv6_addresses
 
     def parsed_addresses(self, version: IPVersion = IPVersion.All) -> List[str]:
         """List addresses in their parsed string form.
@@ -277,7 +290,7 @@ class ServiceInfo(RecordUpdateListener):
         This means the first address will always be the most recently added
         address of the given IP version.
         """
-        return [str(addr) for addr in self.ip_addresses_by_version(version)]
+        return [str(addr) for addr in self._ip_addresses_by_version_value(version.value)]
 
     def parsed_scoped_addresses(self, version: IPVersion = IPVersion.All) -> List[str]:
         """Equivalent to parsed_addresses, with the exception that IPv6 Link-Local
@@ -293,7 +306,7 @@ class ServiceInfo(RecordUpdateListener):
             return self.parsed_addresses(version)
         return [
             f"{addr}%{self.interface_index}" if addr.version == 6 and addr.is_link_local else str(addr)
-            for addr in self.ip_addresses_by_version(version)
+            for addr in self._ip_addresses_by_version_value(version.value)
         ]
 
     def _set_properties(self, properties: Dict) -> None:
@@ -479,16 +492,19 @@ class ServiceInfo(RecordUpdateListener):
         created: Optional[float] = None,
     ) -> List[DNSAddress]:
         """Return matching DNSAddress from ServiceInfo."""
+        name = self.server or self.name
+        ttl = override_ttl if override_ttl is not None else self.host_ttl
+        class_ = _CLASS_IN | _CLASS_UNIQUE
         return [
             DNSAddress(
-                self.server or self.name,
+                name,
                 _TYPE_AAAA if address.version == 6 else _TYPE_A,
-                _CLASS_IN | _CLASS_UNIQUE,
-                override_ttl if override_ttl is not None else self.host_ttl,
+                class_,
+                ttl,
                 address.packed,
                 created=created,
             )
-            for address in self.ip_addresses_by_version(version)
+            for address in self._ip_addresses_by_version_value(version.value)
         ]
 
     def dns_pointer(self, override_ttl: Optional[int] = None, created: Optional[float] = None) -> DNSPointer:
@@ -527,6 +543,35 @@ class ServiceInfo(RecordUpdateListener):
             created,
         )
 
+    def dns_nsec(
+        self, missing_types: List[int], override_ttl: Optional[int] = None, created: Optional[float] = None
+    ) -> DNSNsec:
+        """Return DNSNsec from ServiceInfo."""
+        return DNSNsec(
+            self.name,
+            _TYPE_NSEC,
+            _CLASS_IN | _CLASS_UNIQUE,
+            override_ttl if override_ttl is not None else self.host_ttl,
+            self.name,
+            missing_types,
+            created,
+        )
+
+    def get_address_and_nsec_records(
+        self, override_ttl: Optional[int] = None, created: Optional[float] = None
+    ) -> Set[DNSRecord]:
+        """Build a set of address records and NSEC records for non-present record types."""
+        seen_types: Set[int] = set()
+        records: Set[DNSRecord] = set()
+        for dns_address in self.dns_addresses(override_ttl, IPVersion.All, created):
+            seen_types.add(dns_address.type)
+            records.add(dns_address)
+        missing_types: Set[int] = _ADDRESS_RECORD_TYPES - seen_types
+        if missing_types:
+            assert self.server is not None, "Service server must be set for NSEC record."
+            records.add(self.dns_nsec(list(missing_types), override_ttl, created))
+        return records
+
     def _get_address_records_from_cache_by_type(self, zc: 'Zeroconf', _type: int) -> List[DNSAddress]:
         """Get the addresses from the cache."""
         if self.server_key is None:
@@ -572,7 +617,12 @@ class ServiceInfo(RecordUpdateListener):
         return bool(self.text is not None and (self._ipv4_addresses or self._ipv6_addresses))
 
     def request(
-        self, zc: 'Zeroconf', timeout: float, question_type: Optional[DNSQuestionType] = None
+        self,
+        zc: 'Zeroconf',
+        timeout: float,
+        question_type: Optional[DNSQuestionType] = None,
+        addr: Optional[str] = None,
+        port: int = _MDNS_PORT,
     ) -> bool:
         """Returns true if the service could be discovered on the
         network, and updates this object with details discovered.
@@ -584,13 +634,29 @@ class ServiceInfo(RecordUpdateListener):
         assert zc.loop is not None and zc.loop.is_running()
         if zc.loop == get_running_loop():
             raise RuntimeError("Use AsyncServiceInfo.async_request from the event loop")
-        return bool(run_coro_with_timeout(self.async_request(zc, timeout, question_type), zc.loop, timeout))
+        return bool(
+            run_coro_with_timeout(
+                self.async_request(zc, timeout, question_type, addr, port), zc.loop, timeout
+            )
+        )
 
     async def async_request(
-        self, zc: 'Zeroconf', timeout: float, question_type: Optional[DNSQuestionType] = None
+        self,
+        zc: 'Zeroconf',
+        timeout: float,
+        question_type: Optional[DNSQuestionType] = None,
+        addr: Optional[str] = None,
+        port: int = _MDNS_PORT,
     ) -> bool:
         """Returns true if the service could be discovered on the
         network, and updates this object with details discovered.
+
+        This method will be run in the event loop.
+
+        Passing addr and port is optional, and will default to the
+        mDNS multicast address and port. This is useful for directing
+        requests to a specific host that may be able to respond across
+        subnets.
         """
         if not zc.started:
             await zc.async_wait_for_start()
@@ -614,7 +680,7 @@ class ServiceInfo(RecordUpdateListener):
                     first_request = False
                     if not out.questions:
                         return self.load_from_cache(zc)
-                    zc.async_send(out)
+                    zc.async_send(out, addr, port)
                     next_ = now + delay
                     delay *= 2
                     next_ += random.randint(*_AVOID_SYNC_DELAY_RANDOM_INTERVAL)
